﻿using FlowerShopBusinessLogic.OfficePackage.HelperModels;
using FlowerShopBusinessLogic.OfficePackage;
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;

namespace FlowerShopBusinessLogic.BusinessLogics
{
    public class ReportLogic : IReportLogic
    {
        private readonly IComponentStorage _componentStorage;
        private readonly IFlowerStorage _flowerStorage;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;
        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToWord _saveToWord;
        private readonly AbstractSaveToPdf _saveToPdf;
        public ReportLogic(IComponentStorage componentStorage, IFlowerStorage flowerStorage, IOrderStorage orderStorage, AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf, IShopStorage shopStorage)
		{
			_componentStorage = componentStorage;
			_flowerStorage = flowerStorage;
			_orderStorage = orderStorage;
			_saveToExcel = saveToExcel;
			_saveToWord = saveToWord;
			_saveToPdf = saveToPdf;
			_shopStorage = shopStorage;
		}
		public List<ReportFlowerComponentViewModel> GetFlowerComponent()
        {
            var components = _componentStorage.GetFullList();
            var floweres = _flowerStorage.GetFullList();
            var list = new List<ReportFlowerComponentViewModel>();

            foreach (var flower in floweres)
            {
                var record = new ReportFlowerComponentViewModel
                {
                    FlowerName = flower.FlowerName,
                    Components = new List<Tuple<string, int>>(),
                    TotalCount = 0
                };
                foreach (var component in components)
                {
                    if (flower.FlowerComponents.ContainsKey(component.Id))
                    {
                        record.Components.Add(new Tuple<string, int>(component.ComponentName, flower.FlowerComponents[component.Id].Item2));
                        record.TotalCount += flower.FlowerComponents[component.Id].Item2;
                    }
                }
                list.Add(record);
            }
            return list;
        }

		public List<ReportShopFlowerViewModel> GetShopFlowers()
		{
			var shops = _shopStorage.GetFullList();
			var list = new List<ReportShopFlowerViewModel>();

			foreach (var shop in shops)
			{
				var record = new ReportShopFlowerViewModel
				{
					ShopName = shop.ShopName,
					Flowers = new List<(string Flower, int Count)>(),
					TotalCount = 0,
				};
				foreach (var iceCream in shop.ShopFlowers)
				{
					record.Flowers.Add(new(iceCream.Value.Item1.FlowerName, iceCream.Value.Item2));
					record.TotalCount += iceCream.Value.Item2;
				}
				list.Add(record);
			}
			return list;
		}
		public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            }).Select(x => new ReportOrdersViewModel
            {
                Id = x.Id,
                DateCreate = x.DateCreate,
                FlowerName = x.FlowerName,
                Status = x.Status.ToString(),
                Sum = x.Sum
            }).ToList();
        }

		public List<ReportOrdersByDateViewModel> GetGroupedByDateOrders()
		{
			return _orderStorage.GetFullList().GroupBy(x => x.DateCreate.Date).Select(x => new ReportOrdersByDateViewModel
			{
				Date = x.Key,
				Count = x.Count(),
				Sum = x.Select(y => y.Sum).Sum()
			}).ToList();
		}

		public void SaveComponentsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список платьев",
                Flowers = _flowerStorage.GetFullList()
            });
        }
		public void SaveShopsToWordFile(ReportBindingModel model)
		{
			_saveToWord.CreateShopsTable(new WordInfo
			{
				FileName = model.FileName,
				Title = "Список магазинов",
				Shops = _shopStorage.GetFullList()
			});
		}
		public void SaveFlowerComponentToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Список компонентов",
                FlowerComponents = GetFlowerComponent()
            });
        }
		public void SaveShopFlowerToExcelFile(ReportBindingModel model)
		{
			_saveToExcel.CreateShopReport(new ExcelInfo
			{
				FileName = model.FileName,
				Title = "Загруженность магазинов",
				ShopFlowers = GetShopFlowers()
			});
		}
		public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }
		public void SaveGroupedOrdersToPdfFile(ReportBindingModel model)
		{
			_saveToPdf.CreateDocWithGroupedOrders(new PdfInfo
			{
				FileName = model.FileName,
				Title = "Заказы по датам",
				GroupedOrders = GetGroupedByDateOrders()
			});
		}
	}
}
